# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class TypeAliases(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = TypeAliases()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsTypeAliases(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def TypeAliasesBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4D\x4F\x4E\x53", size_prefixed=size_prefixed)

    # TypeAliases
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # TypeAliases
    def I8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def U8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def I16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int16Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def U16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def I32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def U32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def I64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def U64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # TypeAliases
    def F32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return 0.0

    # TypeAliases
    def F64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # TypeAliases
    def V8(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Int8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # TypeAliases
    def V8AsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Int8Flags, o)
        return 0

    # TypeAliases
    def V8Length(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # TypeAliases
    def V8IsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        return o == 0

    # TypeAliases
    def Vf64(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # TypeAliases
    def Vf64AsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # TypeAliases
    def Vf64Length(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # TypeAliases
    def Vf64IsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        return o == 0

def TypeAliasesStart(builder):
    builder.StartObject(12)

def Start(builder):
    TypeAliasesStart(builder)

def TypeAliasesAddI8(builder, i8):
    builder.PrependInt8Slot(0, i8, 0)

def AddI8(builder, i8):
    TypeAliasesAddI8(builder, i8)

def TypeAliasesAddU8(builder, u8):
    builder.PrependUint8Slot(1, u8, 0)

def AddU8(builder, u8):
    TypeAliasesAddU8(builder, u8)

def TypeAliasesAddI16(builder, i16):
    builder.PrependInt16Slot(2, i16, 0)

def AddI16(builder, i16):
    TypeAliasesAddI16(builder, i16)

def TypeAliasesAddU16(builder, u16):
    builder.PrependUint16Slot(3, u16, 0)

def AddU16(builder, u16):
    TypeAliasesAddU16(builder, u16)

def TypeAliasesAddI32(builder, i32):
    builder.PrependInt32Slot(4, i32, 0)

def AddI32(builder, i32):
    TypeAliasesAddI32(builder, i32)

def TypeAliasesAddU32(builder, u32):
    builder.PrependUint32Slot(5, u32, 0)

def AddU32(builder, u32):
    TypeAliasesAddU32(builder, u32)

def TypeAliasesAddI64(builder, i64):
    builder.PrependInt64Slot(6, i64, 0)

def AddI64(builder, i64):
    TypeAliasesAddI64(builder, i64)

def TypeAliasesAddU64(builder, u64):
    builder.PrependUint64Slot(7, u64, 0)

def AddU64(builder, u64):
    TypeAliasesAddU64(builder, u64)

def TypeAliasesAddF32(builder, f32):
    builder.PrependFloat32Slot(8, f32, 0.0)

def AddF32(builder, f32):
    TypeAliasesAddF32(builder, f32)

def TypeAliasesAddF64(builder, f64):
    builder.PrependFloat64Slot(9, f64, 0.0)

def AddF64(builder, f64):
    TypeAliasesAddF64(builder, f64)

def TypeAliasesAddV8(builder, v8):
    builder.PrependUOffsetTRelativeSlot(10, flatbuffers.number_types.UOffsetTFlags.py_type(v8), 0)

def AddV8(builder, v8):
    TypeAliasesAddV8(builder, v8)

def TypeAliasesStartV8Vector(builder, numElems):
    return builder.StartVector(1, numElems, 1)

def StartV8Vector(builder, numElems):
    return TypeAliasesStartV8Vector(builder, numElems)

def TypeAliasesAddVf64(builder, vf64):
    builder.PrependUOffsetTRelativeSlot(11, flatbuffers.number_types.UOffsetTFlags.py_type(vf64), 0)

def AddVf64(builder, vf64):
    TypeAliasesAddVf64(builder, vf64)

def TypeAliasesStartVf64Vector(builder, numElems):
    return builder.StartVector(8, numElems, 8)

def StartVf64Vector(builder, numElems):
    return TypeAliasesStartVf64Vector(builder, numElems)

def TypeAliasesEnd(builder):
    return builder.EndObject()

def End(builder):
    return TypeAliasesEnd(builder)

try:
    from typing import List
except:
    pass

class TypeAliasesT(object):

    # TypeAliasesT
    def __init__(
        self,
        i8 = 0,
        u8 = 0,
        i16 = 0,
        u16 = 0,
        i32 = 0,
        u32 = 0,
        i64 = 0,
        u64 = 0,
        f32 = 0.0,
        f64 = 0.0,
        v8 = None,
        vf64 = None,
    ):
        self.i8 = i8  # type: int
        self.u8 = u8  # type: int
        self.i16 = i16  # type: int
        self.u16 = u16  # type: int
        self.i32 = i32  # type: int
        self.u32 = u32  # type: int
        self.i64 = i64  # type: int
        self.u64 = u64  # type: int
        self.f32 = f32  # type: float
        self.f64 = f64  # type: float
        self.v8 = v8  # type: Optional[List[int]]
        self.vf64 = vf64  # type: Optional[List[float]]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        typeAliases = TypeAliases()
        typeAliases.Init(buf, pos)
        return cls.InitFromObj(typeAliases)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, typeAliases):
        x = TypeAliasesT()
        x._UnPack(typeAliases)
        return x

    # TypeAliasesT
    def _UnPack(self, typeAliases):
        if typeAliases is None:
            return
        self.i8 = typeAliases.I8()
        self.u8 = typeAliases.U8()
        self.i16 = typeAliases.I16()
        self.u16 = typeAliases.U16()
        self.i32 = typeAliases.I32()
        self.u32 = typeAliases.U32()
        self.i64 = typeAliases.I64()
        self.u64 = typeAliases.U64()
        self.f32 = typeAliases.F32()
        self.f64 = typeAliases.F64()
        if not typeAliases.V8IsNone():
            if np is None:
                self.v8 = []
                for i in range(typeAliases.V8Length()):
                    self.v8.append(typeAliases.V8(i))
            else:
                self.v8 = typeAliases.V8AsNumpy()
        if not typeAliases.Vf64IsNone():
            if np is None:
                self.vf64 = []
                for i in range(typeAliases.Vf64Length()):
                    self.vf64.append(typeAliases.Vf64(i))
            else:
                self.vf64 = typeAliases.Vf64AsNumpy()

    # TypeAliasesT
    def Pack(self, builder):
        if self.v8 is not None:
            if np is not None and type(self.v8) is np.ndarray:
                v8 = builder.CreateNumpyVector(self.v8)
            else:
                TypeAliasesStartV8Vector(builder, len(self.v8))
                for i in reversed(range(len(self.v8))):
                    builder.PrependByte(self.v8[i])
                v8 = builder.EndVector()
        if self.vf64 is not None:
            if np is not None and type(self.vf64) is np.ndarray:
                vf64 = builder.CreateNumpyVector(self.vf64)
            else:
                TypeAliasesStartVf64Vector(builder, len(self.vf64))
                for i in reversed(range(len(self.vf64))):
                    builder.PrependFloat64(self.vf64[i])
                vf64 = builder.EndVector()
        TypeAliasesStart(builder)
        TypeAliasesAddI8(builder, self.i8)
        TypeAliasesAddU8(builder, self.u8)
        TypeAliasesAddI16(builder, self.i16)
        TypeAliasesAddU16(builder, self.u16)
        TypeAliasesAddI32(builder, self.i32)
        TypeAliasesAddU32(builder, self.u32)
        TypeAliasesAddI64(builder, self.i64)
        TypeAliasesAddU64(builder, self.u64)
        TypeAliasesAddF32(builder, self.f32)
        TypeAliasesAddF64(builder, self.f64)
        if self.v8 is not None:
            TypeAliasesAddV8(builder, v8)
        if self.vf64 is not None:
            TypeAliasesAddVf64(builder, vf64)
        typeAliases = TypeAliasesEnd(builder)
        return typeAliases



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Stat(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Stat()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsStat(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def StatBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4D\x4F\x4E\x53", size_prefixed=size_prefixed)

    # Stat
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Stat
    def Id(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Stat
    def Val(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # Stat
    def Count(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

def StatStart(builder):
    builder.StartObject(3)

def Start(builder):
    StatStart(builder)

def StatAddId(builder, id):
    builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(id), 0)

def AddId(builder, id):
    StatAddId(builder, id)

def StatAddVal(builder, val):
    builder.PrependInt64Slot(1, val, 0)

def AddVal(builder, val):
    StatAddVal(builder, val)

def StatAddCount(builder, count):
    builder.PrependUint16Slot(2, count, 0)

def AddCount(builder, count):
    StatAddCount(builder, count)

def StatEnd(builder):
    return builder.EndObject()

def End(builder):
    return StatEnd(builder)


class StatT(object):

    # StatT
    def __init__(
        self,
        id = None,
        val = 0,
        count = 0,
    ):
        self.id = id  # type: Optional[str]
        self.val = val  # type: int
        self.count = count  # type: int

    @classmethod
    def InitFromBuf(cls, buf, pos):
        stat = Stat()
        stat.Init(buf, pos)
        return cls.InitFromObj(stat)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, stat):
        x = StatT()
        x._UnPack(stat)
        return x

    # StatT
    def _UnPack(self, stat):
        if stat is None:
            return
        self.id = stat.Id()
        self.val = stat.Val()
        self.count = stat.Count()

    # StatT
    def Pack(self, builder):
        if self.id is not None:
            id = builder.CreateString(self.id)
        StatStart(builder)
        if self.id is not None:
            StatAddId(builder, id)
        StatAddVal(builder, self.val)
        StatAddCount(builder, self.count)
        stat = StatEnd(builder)
        return stat



# Copyright 2013-2025 The py-lmdb authors, all rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted only as authorized by the OpenLDAP
# Public License.
#
# A copy of this license is available in the file LICENSE in the
# top-level directory of the distribution or, alternatively, at
# <http://www.OpenLDAP.org/license.html>.
#
# OpenLDAP is a registered trademark of the OpenLDAP Foundation.
#
# Individual files and/or contributed packages may be copyright by
# other parties and/or subject to additional restrictions.
#
# This work also contains materials derived from public sources.
#
# Additional information about OpenLDAP can be obtained at
# <http://www.openldap.org/>.

"""
Python wrapper for OpenLDAP's "Lightning" MDB database.

Please see https://lmdb.readthedocs.io/
"""


def _reading_docs():
    """Return True if Sphinx is currently parsing this file."""
    return "sphinx" in __import__("sys").modules


# CFFI-only mode (bundled with zlmdb)
from zlmdb._lmdb_vendor.cffi import *
from zlmdb._lmdb_vendor.cffi import open
from zlmdb._lmdb_vendor.cffi import __all__
from zlmdb._lmdb_vendor.cffi import __doc__



# Copyright 2013-2025 The py-lmdb authors, all rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted only as authorized by the OpenLDAP
# Public License.
#
# A copy of this license is available in the file LICENSE in the
# top-level directory of the distribution or, alternatively, at
# <http://www.OpenLDAP.org/license.html>.
#
# OpenLDAP is a registered trademark of the OpenLDAP Foundation.
#
# Individual files and/or contributed packages may be copyright by
# other parties and/or subject to additional restrictions.
#
# This work also contains materials derived from public sources.
#
# Additional information about OpenLDAP can be obtained at
# <http://www.openldap.org/>.

import zlmdb.lmdb as lmdb

# Open (and create if necessary) our database environment. Must specify
# max_dbs=... since we're opening subdbs.
env = lmdb.open("/tmp/address-book.lmdb", max_dbs=10)

# Now create subdbs for home and business addresses.
home_db = env.open_db(b"home")
business_db = env.open_db(b"business")


# Add some telephone numbers to each DB:
with env.begin(write=True) as txn:
    txn.put(b"mum", b"012345678", db=home_db)
    txn.put(b"dad", b"011232211", db=home_db)
    txn.put(b"dentist", b"044415121", db=home_db)
    txn.put(b"hospital", b"078126321", db=home_db)

    txn.put(b"vendor", b"0917465628", db=business_db)
    txn.put(b"customer", b"0553211232", db=business_db)
    txn.put(b"coworker", b"0147652935", db=business_db)
    txn.put(b"boss", b"0123151232", db=business_db)
    txn.put(b"manager", b"0644810485", db=business_db)


# Iterate each DB to show the keys are sorted:
with env.begin() as txn:
    for name, db in ("home", home_db), ("business", business_db):
        print("DB:", name)
        for key, value in txn.cursor(db=db):
            print("  ", key, value)
        print()


# Now let's update some phone numbers. We can specify the default subdb when
# starting the transaction, rather than pass it in every time:
with env.begin(write=True, db=home_db) as txn:
    print("Updating number for dentist")
    txn.put(b"dentist", b"099991231")

    print("Deleting number for hospital")
    txn.delete(b"hospital")
    print()

    print("Home DB is now:")
    for key, value in txn.cursor():
        print("  ", key, value)
    print()


# Now let's look up a number in the business DB
with env.begin(db=business_db) as txn:
    print("Boss telephone number:", txn.get(b"boss"))
    print()


# We got fired, time to delete all keys from the business DB.
with env.begin(write=True) as txn:
    print("Deleting all numbers from business DB:")
    txn.drop(business_db, delete=False)

    print("Adding number for recruiter to business DB")
    txn.put(b"recruiter", b"04123125324", db=business_db)

    print("Business DB is now:")
    for key, value in txn.cursor(db=business_db):
        print("  ", key, value)
    print()



# Generated by the gRPC FlatBuffers compiler. DO NOT EDIT!

from __future__ import annotations

import grpc
import typing

from service_test_generated import HelloRequest, HelloResponse


class HelloServiceStub(object):
  def __init__(self, channel: grpc.Channel) -> None: ...
  def Hello(self, request: HelloRequest) -> HelloResponse: ...
  def StreamClient(self, request_iterator: typing.Iterator[HelloRequest]) -> HelloResponse: ...
  def StreamServer(self, request: HelloRequest) -> typing.Iterator[HelloResponse]: ...
  def Stream(self, request_iterator: typing.Iterator[HelloRequest]) -> typing.Iterator[HelloResponse]: ...


class HelloServiceServicer(object):
  def Hello(self, request: HelloRequest, context: grpc.ServicerContext) -> HelloResponse: ...
  def StreamClient(self, request_iterator: typing.Iterator[HelloRequest], context: grpc.ServicerContext) -> HelloResponse: ...
  def StreamServer(self, request: HelloRequest, context: grpc.ServicerContext) -> typing.Iterator[HelloResponse]: ...
  def Stream(self, request_iterator: typing.Iterator[HelloRequest], context: grpc.ServicerContext) -> typing.Iterator[HelloResponse]: ...


def add_HelloServiceServicer_to_server(servicer: HelloServiceServicer, server: grpc.Server) -> None: ...




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: reflection

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class KeyValue(object):
    __slots__ = ["_tab"]

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = KeyValue()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsKeyValue(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)

    @classmethod
    def KeyValueBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(
            buf, offset, b"\x42\x46\x42\x53", size_prefixed=size_prefixed
        )

    # KeyValue
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # KeyValue
    def Key(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # KeyValue
    def Value(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None


def KeyValueStart(builder):
    builder.StartObject(2)


def Start(builder):
    return KeyValueStart(builder)


def KeyValueAddKey(builder, key):
    builder.PrependUOffsetTRelativeSlot(
        0, flatbuffers.number_types.UOffsetTFlags.py_type(key), 0
    )


def AddKey(builder, key):
    return KeyValueAddKey(builder, key)


def KeyValueAddValue(builder, value):
    builder.PrependUOffsetTRelativeSlot(
        1, flatbuffers.number_types.UOffsetTFlags.py_type(value), 0
    )


def AddValue(builder, value):
    return KeyValueAddValue(builder, value)


def KeyValueEnd(builder):
    return builder.EndObject()


def End(builder):
    return KeyValueEnd(builder)



# Generated by the gRPC FlatBuffers compiler. DO NOT EDIT!

import flatbuffers
import grpc
from service_test_generated import HelloRequest, HelloResponse


def _serialize_to_bytes(table):
  buf = table._tab.Bytes
  n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, 0)
  if table._tab.Pos != n:
    raise ValueError('must be a top-level table')
  return bytes(buf)


class HelloServiceStub(object):
  """Interface exported by the server."""

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """

    self.Hello = channel.unary_unary(
        method='/example.HelloService/Hello',
        request_serializer=_serialize_to_bytes,
        response_deserializer=HelloResponse.GetRootAs,
    )

    self.StreamClient = channel.stream_unary(
        method='/example.HelloService/StreamClient',
        request_serializer=_serialize_to_bytes,
        response_deserializer=HelloResponse.GetRootAs,
    )

    self.StreamServer = channel.unary_stream(
        method='/example.HelloService/StreamServer',
        request_serializer=_serialize_to_bytes,
        response_deserializer=HelloResponse.GetRootAs,
    )

    self.Stream = channel.stream_stream(
        method='/example.HelloService/Stream',
        request_serializer=_serialize_to_bytes,
        response_deserializer=HelloResponse.GetRootAs,
    )


class HelloServiceServicer(object):
  """Interface exported by the server."""

  def Hello(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamClient(self, request_iterator, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StreamServer(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Stream(self, request_iterator, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_HelloServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Hello': grpc.unary_unary_rpc_method_handler(
          servicer.Hello,
          request_deserializer=HelloRequest.GetRootAs,
          response_serializer=_serialize_to_bytes,
      ),
      'StreamClient': grpc.stream_unary_rpc_method_handler(
          servicer.StreamClient,
          request_deserializer=HelloRequest.GetRootAs,
          response_serializer=_serialize_to_bytes,
      ),
      'StreamServer': grpc.unary_stream_rpc_method_handler(
          servicer.StreamServer,
          request_deserializer=HelloRequest.GetRootAs,
          response_serializer=_serialize_to_bytes,
      ),
      'Stream': grpc.stream_stream_rpc_method_handler(
          servicer.Stream,
          request_deserializer=HelloRequest.GetRootAs,
          response_serializer=_serialize_to_bytes,
      ),
  }

  generic_handler = grpc.method_handlers_generic_handler(
      'example.HelloService', rpc_method_handlers
  )

  server.add_generic_rpc_handlers((generic_handler,))



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.ArrayStruct import ArrayStruct, ArrayStructT

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class ArrayTable(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> ArrayTable: ...
  @classmethod
  def GetRootAsArrayTable(cls, buf: bytes, offset: int) -> ArrayTable: ...
  @classmethod
  def ArrayTableBufferHasIdentifier(cls, buf: bytes, offset: int, size_prefixed: bool) -> bool: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
  def A(self) -> ArrayStruct | None: ...
class ArrayTableT(object):
  a: ArrayStructT | None
  def __init__(
    self,
    a: 'ArrayStructT' | None = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> ArrayTableT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> ArrayTableT: ...
  @classmethod
  def InitFromObj(cls, arrayTable: ArrayTable) -> ArrayTableT: ...
  def _UnPack(self, arrayTable: ArrayTable) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...
def ArrayTableStart(builder: flatbuffers.Builder) -> None: ...
def Start(builder: flatbuffers.Builder) -> None: ...
def ArrayTableAddA(builder: flatbuffers.Builder, a: uoffset) -> None: ...
def ArrayTableEnd(builder: flatbuffers.Builder) -> uoffset: ...
def End(builder: flatbuffers.Builder) -> uoffset: ...




# Copyright 2022 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from flatc_test import *


class CppTests:

  def Flatten(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", "foo.fbs"])

    # Foo should be generated in place and include bar flatten
    assert_file_and_contents("foo_generated.h", '#include "bar_generated.h"')

  def FlattenAbsolutePath(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", make_absolute("foo.fbs")])

    # Foo should be generated in place and include bar flatten
    assert_file_and_contents("foo_generated.h", '#include "bar_generated.h"')

  def FlattenSubDirectory(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", "bar/bar.fbs"])

    # Bar should be generated in place and include baz
    assert_file_and_contents("bar_generated.h", '#include "baz_generated.h"')

  def FlattenOutPath(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", "-o", ".tmp", "foo.fbs"])

    # Foo should be generated in the out path and include bar flatten to the out path.
    assert_file_and_contents(
        ".tmp/foo_generated.h", '#include "bar_generated.h"'
    )

  def FlattenOutPathSuperDirectory(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", "-o", "../.tmp", "foo.fbs"])

    # Foo should be generated in the out path and include bar flatten to the out path.
    assert_file_and_contents(
        "../.tmp/foo_generated.h", '#include "bar_generated.h"'
    )

  def FlattenOutPathSubDirectory(self):
    # Generate just foo with a "flatten" import of bar.
    flatc(["--cpp", "-o", ".tmp", "bar/bar.fbs"])

    # Bar should be generated in the out path and include baz flatten to the out path.
    assert_file_and_contents(
        ".tmp/bar_generated.h", '#include "baz_generated.h"'
    )

  def KeepPrefix(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "foo.fbs"])

    assert_file_and_contents(
        "foo_generated.h", '#include "bar/bar_generated.h"'
    )

  def KeepPrefixAbsolutePath(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", make_absolute("foo.fbs")])

    assert_file_and_contents(
        "foo_generated.h", '#include "bar/bar_generated.h"'
    )

  def KeepPrefixSubDirectory(self):
    # Generate with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "bar/bar.fbs"])

    assert_file_and_contents(
        "bar_generated.h", '#include "baz/baz_generated.h"'
    )

  def KeepPrefixOutPath(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "-o", ".tmp", "foo.fbs"])

    assert_file_and_contents(
        ".tmp/foo_generated.h",
        '#include "bar/bar_generated.h"',
    )

  def KeepPrefixOutPathSubDirectory(self):
    # Generate with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "-o", ".tmp", "bar/bar.fbs"])

    assert_file_and_contents(
        ".tmp/bar_generated.h", '#include "baz/baz_generated.h"'
    )

  def IncludePrefix(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--include-prefix", "test", "foo.fbs"])

    assert_file_and_contents(
        "foo_generated.h", '#include "test/bar_generated.h"'
    )

  def IncludePrefixAbolutePath(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--include-prefix", "test", make_absolute("foo.fbs")])

    assert_file_and_contents(
        "foo_generated.h", '#include "test/bar_generated.h"'
    )

  def IncludePrefixSubDirectory(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--include-prefix", "test", "bar/bar.fbs"])

    assert_file_and_contents(
        "bar_generated.h", '#include "test/baz_generated.h"'
    )

  def IncludePrefixOutPath(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--include-prefix", "test", "-o", ".tmp", "foo.fbs"])

    assert_file_and_contents(
        ".tmp/foo_generated.h", '#include "test/bar_generated.h"'
    )

  def IncludePrefixOutPathSubDirectory(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--include-prefix", "test", "-o", ".tmp", "bar/bar.fbs"])

    assert_file_and_contents(
        ".tmp/bar_generated.h", '#include "test/baz_generated.h"'
    )

  def KeepPrefixIncludePrefix(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "--include-prefix", "test", "foo.fbs"])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        "foo_generated.h", '#include "test/bar/bar_generated.h"'
    )

  def KeepPrefixIncludePrefixAbsolutePath(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc([
        "--cpp",
        "--keep-prefix",
        "--include-prefix",
        "test",
        make_absolute("foo.fbs"),
    ])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        "foo_generated.h", '#include "test/bar/bar_generated.h"'
    )

  def KeepPrefixIncludePrefixSubDirectory(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc(["--cpp", "--keep-prefix", "--include-prefix", "test", "bar/bar.fbs"])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        "bar_generated.h", '#include "test/baz/baz_generated.h"'
    )

  def KeepPrefixIncludePrefixOutPathSubDirectory(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc([
        "--cpp",
        "--keep-prefix",
        "--include-prefix",
        "test",
        "-o",
        ".tmp",
        "bar/bar.fbs",
    ])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        ".tmp/bar_generated.h", '#include "test/baz/baz_generated.h"'
    )

  def KeepPrefixIncludePrefixOutPathSuperDirectory(self):
    # Generate just foo with the import of bar keeping the prefix of where it is located.
    flatc([
        "--cpp",
        "--keep-prefix",
        "--include-prefix",
        "test",
        "-o",
        "../.tmp",
        "bar/bar.fbs",
    ])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        "../.tmp/bar_generated.h", '#include "test/baz/baz_generated.h"'
    )

  def KeepPrefixIncludePrefixoutPathAbsoluePaths_SuperDirectoryReference(self):
    # Generate bar_with_foo that references a type in a super directory.
    flatc([
        "--cpp",
        "--keep-prefix",
        "--include-prefix",
        "generated",
        "-I",
        str(script_path.absolute()),
        "-o",
        str(Path(script_path, ".tmp").absolute()),
        str(Path(script_path, "bar/bar_with_foo.fbs").absolute()),
    ])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        ".tmp/bar_with_foo_generated.h",
        [
            '#include "generated/baz/baz_generated.h"',
            '#include "generated/foo_generated.h"',
        ],
    )

  def KeepPrefixIncludePrefixoutPath_SuperDirectoryReference(self):
    # Generate bar_with_foo that references a type in a super directory.
    flatc([
        "--cpp",
        "--keep-prefix",
        "--include-prefix",
        "generated",
        "-I",
        "./",
        "-o",
        ".tmp",
        "bar/bar_with_foo.fbs",
    ])

    # The include prefix should come first, with the kept prefix next.
    assert_file_and_contents(
        ".tmp/bar_with_foo_generated.h",
        [
            '#include "generated/baz/baz_generated.h"',
            '#include "generated/foo_generated.h"',
        ],
        unlink=False,
    )
